/****************************************************************
 * Skyler Schneider ss868                                       *
 * Jack Ellsworth   jfe5                                        *
 * ECE 4760 Final Project                                       *
 * USB Wireless Tilt Mouse                                      *
 ****************************************************************/

// Receiver unit consists of USB and wireless RX.
// This file receives data wirelessly and sends it to the USB MCU.

/****************************************************************
 * Hardware Connections                                         *
 ****************************************************************/

/*
 * PORTB
 *  B.0 - Data to USB MCU
 *  B.1 - Data to USB MCU
 *  B.2 - Data to USB MCU
 *  B.3 - Data to USB MCU
 *  B.5 - Addr to USB MCU
 *  B.6 - Addr to USB MCU
 *  B.7 - Addr to USB MCU
 */

/*
 * PORTD
 *  D.0 - Wireless receiver
 *  D.5 - ACK from USB MCU
 *  D.6 - ACK from USB MCU
 *  D.7 - ACK from USB MCU
 */

/****************************************************************
 * Compiler Directives                                          *
 ****************************************************************/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "txrx.c"

// Wireless definitions
#define MAX_RX_LENGTH 32
#define data_buffer   32
#define threshold     2000

/****************************************************************
 * Global Declarations                                          *
 ****************************************************************/

// Structure for storing mouse status data to be sent to USB MCU
typedef struct {
    char buttonMask;
    char dxL;
    char dxH;
    char dyL;
    char dyH;
    char dWheelL;
    char dWheelH;
} txrx_usb_buf_t;

txrx_usb_buf_t rx_usb_buf;

// Wireless variables
char     my_rx_data[MAX_RX_LENGTH];
char     length;
uint16_t msSinceLastData;

// Timer1 variables for 1ms period
uint16_t TCNT1_prev;
uint16_t TCNT1_current;
uint16_t TCNT1_change;

void init(void); // initialization method

/****************************************************************
 * Main Routine                                                 *
 ****************************************************************/

int main(void) {
    
    int i;
    int cntr = 0;
    int msCntr = 0;
    uint8_t receivedData;
    
    init();
    
    while(1) {
        
        // Check if received data from wireless receiver
        receivedData = 0;
        if(rxdone() == 1) {
            // Yes, received wireless data - decode it
            i = 0;
            init_getrx();
            while(rx_empty() != 1) {
                my_rx_data[i] = get_next_rx_data();
                i++;
            }
            
            // 0 - 170
            // 1 - start_char
            // 2 - ID
            // 3 - length
            // 4 to 12 - payload
            rx_reset(MAX_RX_LENGTH);
            if(my_rx_data[4] == 6) {
                rx_usb_buf.buttonMask = 0b00100000 | (my_rx_data[5]  & 0x0F);
                rx_usb_buf.dxL        = 0b01000000 | (my_rx_data[6]  & 0x0F);
                rx_usb_buf.dxH        = 0b01100000 | (my_rx_data[7]  & 0x0F);
                rx_usb_buf.dyL        = 0b10000000 | (my_rx_data[8]  & 0x0F);
                rx_usb_buf.dyH        = 0b10100000 | (my_rx_data[9]  & 0x0F);
                rx_usb_buf.dWheelL    = 0b11000000 | (my_rx_data[10] & 0x0F);
                rx_usb_buf.dWheelH    = 0b11100000 | (my_rx_data[11] & 0x0F);
            }
            receivedData = 1;
        }
        
        TCNT1_current = TCNT1;
        TCNT1_change  = TCNT1_current - TCNT1_prev;
        if(TCNT1_change >= 250) {
            // Code runs every 1ms
            TCNT1_prev = TCNT1_current;
            msCntr++;
            if(receivedData) {
                msSinceLastData = 0;
            }
            else {
                msSinceLastData++;
                if(msSinceLastData == 1000) {
                    // If no packets received in past second, zero mouse
                    rx_usb_buf.buttonMask = 0b00100000 | 0x00;
                    rx_usb_buf.dxL        = 0b01000000 | 0x00;
                    rx_usb_buf.dxH        = 0b01100000 | 0x00;
                    rx_usb_buf.dyL        = 0b10000000 | 0x00;
                    rx_usb_buf.dyH        = 0b10100000 | 0x00;
                    rx_usb_buf.dWheelL    = 0b11000000 | 0x00;
                    rx_usb_buf.dWheelH    = 0b11100000 | 0x00;
                }
            }
            if(msCntr >= 500) {
                // Code runs every 500ms - blink LED at 1Hz
                msCntr = 0;
                PORTD = PORTD ^ 0b00000100;
            }
        }
        // Check if USB MCU has received data
        if((PORTB & 0b11100000) == (PIND & 0b11100000)) {
            switch(cntr) {
                case 0:
                    PORTB = rx_usb_buf.buttonMask;
                    cntr++;
                    break;
                case 1:
                    PORTB = rx_usb_buf.dxL;
                    cntr++;
                    break;
                case 2:
                    PORTB = rx_usb_buf.dxH;
                    cntr++;
                    break;
                case 3:
                    PORTB = rx_usb_buf.dyL;
                    cntr++;
                    break;
                case 4:
                    PORTB = rx_usb_buf.dyH;
                    cntr++;
                    break;
                case 5:
                    PORTB = rx_usb_buf.dWheelL;
                    cntr++;
                    break;
                case 6:
                    PORTB = rx_usb_buf.dWheelH;
                    cntr = 0;
                    break;
            }
        }
    }
}

/****************************************************************
 * Initialization Routine                                       *
 ****************************************************************/

void init() {
    
    // Set up ports
    DDRB = 0b11111111; // PORT B is an output
    DDRD = 0b00000100; // LED
    
    // Timer1 runs at prescalar 64
    TCCR1B = 0x03; // Timer1 runs at prescalar 64
    TCNT1_current = TCNT1;
    TCNT1_prev    = TCNT1_current;
    TCNT1_change  = 0;
    
    // Initialize variables
    rx_usb_buf.buttonMask = 0;
    rx_usb_buf.dxL        = 0;
    rx_usb_buf.dxH        = 0;
    rx_usb_buf.dyL        = 0;
    rx_usb_buf.dyH        = 0;
    rx_usb_buf.dWheelL    = 0;
    rx_usb_buf.dWheelH    = 0;
    
    // TXRX init
    txrx_init(0,1,249,0); // RX only - 4000 baud - led off
    rx_reset(MAX_RX_LENGTH);
    msSinceLastData = 0;
    
    // Enable interrupts
    sei();
    
}
